/**
 * @defgroup Playlists Playlists
 */

 /**
 * @defgroup PlaylistHelper Utilities
 * @brief Classes for handling playlist DB saving and loading
 * @ingroup Playlists
 */

/**
 * @defgroup GuiPlaylists Gui Classes
 * @ingroup Playlists
 */
