/* GUI_Podcasts.h */

/* Copyright (C) 2011-2024 Michael Lugmair (Lucio Carreras)
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GUI_PODCASTS_H
#define GUI_PODCASTS_H

#include "AbstractStationPlugin.h"

UI_FWD(GUI_Podcasts)

class PodcastHandler;
class GUI_Podcasts :
	public Gui::AbstractStationPlugin
{
	Q_OBJECT
	PIMPL(GUI_Podcasts)
	UI_CLASS(GUI_Podcasts)

	public:
		GUI_Podcasts(Playlist::Creator* playlistCreator, PodcastHandler* podcastHandler, QWidget* parent = nullptr);
		~GUI_Podcasts() override;

		[[nodiscard]] QString name() const override;
		[[nodiscard]] QString displayName() const override;

	protected:
		void skinChanged() override;

	private slots:
		void searchPodcastTriggered();
		void podcastSelected(const QString& name, const QString& url, bool save);

	private: // NOLINT(*-redundant-access-specifiers)
		void initUi() override;
		void retranslate() override;

		[[nodiscard]] QString titleFallbackName() const override;
		[[nodiscard]] QComboBox* comboStream() override;
		[[nodiscard]] QPushButton* btnPlay() override;
		[[nodiscard]] Gui::MenuToolButton* btnMenu() override;
		[[nodiscard]] GUI_ConfigureStation* createConfigDialog() override;
};

#endif // GUI_PODCASTS_H
