/**
 * @defgroup Covers Covers
 * @brief Contains everythin about cover management. Logic and UI
 */

 /**
 * @defgroup GuiCovers Gui Classes
 * @brief UI side of cover management
 * @ingroup Covers
 */
