/* CustomMimeData.h */

/* Copyright (C) 2011-2024 Michael Lugmair (Lucio Carreras)
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SAYONARA_CUSTOM_MIME_DATA_H
#define SAYONARA_CUSTOM_MIME_DATA_H

#include "Utils/Pimpl.h"

#include <QMimeData>

namespace Gui
{
	class AsyncDropHandler;
	class CustomMimeData :
		public QMimeData
	{
		PIMPL(CustomMimeData)

		public:
			explicit CustomMimeData(const QObject* dragSource = nullptr);
			~CustomMimeData() override;

			void setMetadata(const MetaDataList& tracks);
			[[nodiscard]] const MetaDataList& metadata() const;
			[[nodiscard]] bool hasMetadata() const;

			void setPlaylistSourceIndex(int playlistIndex);
			[[nodiscard]] int playlistSourceIndex() const;

			[[nodiscard]] QString coverUrl() const;
			void setCoverUrl(const QString& url);

			void setAsyncDropHandler(Gui::AsyncDropHandler* handler);
			[[nodiscard]] Gui::AsyncDropHandler* asyncDropHandler() const;

			bool hasDragSource(const QObject* classInstance) const;
	};
}

#endif // SAYONARA_CUSTOM_MIME_DATA_H
