/* Lyrics.h, (Created on 06.01.2024) */

/* Copyright (C) 2011-2024 Michael Lugmair
 *
 * This file is part of Sayonara Player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef SAYONARA_PLAYER_LYRICS_H
#define SAYONARA_PLAYER_LYRICS_H

#include "MP4Frame.h"

#include <QString>

namespace MP4
{
	class Lyrics :
		public MP4Frame<QString>
	{
		public:
			explicit Lyrics(TagLib::MP4::Tag* tag);
			~Lyrics() override = default;

		protected:
			[[nodiscard]] std::optional<QString> mapItemToData(const TagLib::MP4::Item& item) const override;
			[[nodiscard]] std::optional<TagLib::MP4::Item> mapDataToItem(const QString& model) override;
			[[nodiscard]] std::optional<QString> findDataInPropertyMap() const override;
			[[nodiscard]] QByteArray propertyMapKey() const override;
	};
}

#endif //SAYONARA_PLAYER_LYRICS_H
