/* LFMGlobals.h

 * Copyright (C) 2011-2024 Michael Lugmair (Lucio Carreras)
 *
 * This file is part of sayonara-player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * created by Michael Lugmair (Lucio Carreras), 
 * Jul 18, 2012 
 *
 */

#ifndef SAYONARA_LASTFM_GLOBALS_H
#define SAYONARA_LASTFM_GLOBALS_H

namespace LastFM
{
	constexpr const auto* ApiKey = "51d6f9eaef806f603f346844bef326ba";
	constexpr const auto* ApiSecret = "1093d769e54858cb0d21d42b35a8f603";
	constexpr const auto* BaseUrl = "https://ws.audioscrobbler.com/2.0/";
}

#endif /* SAYONARA_LASTFM_GLOBALS_H */
