
/**
 * @defgroup Interfaces Interfaces
 */

/**
 * @defgroup Language Language
 * @brief methods to keep convenience between translations
 */


 /**
 * @defgroup RemoteControl Remote control
 * @brief remote controlling Sayonara via TCP protocol
 */

/**
 * @defgroup Streams Streams
 * @brief Everything about streaming. SomaFM, Soundcloud
 * web radio and podcasts
 */

/**
 * @defgroup Database Database
 * @brief Database classes and their relations
 */

