/**
 * @defgroup Library Library
 * @brief Basic library functions
 */

/**
 * @defgroup LibraryPlugins Library Plugins
 * @brief External library view handling
 * @ingroup Library
 */

/**
 * @defgroup GuiLibrary Gui Classes
 * @brief UI side of library view
 * @ingroup Library
 */
