/* ShortcutIdentifier.h */

/* Copyright (C) 2011-2024 Michael Lugmair (Lucio Carreras)
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SHORTCUTIDENTIFIER_H
#define SHORTCUTIDENTIFIER_H

enum ShortcutIdentifier
{
	AddTab = 0,
	AlbumArtists,
	Append,
	ClosePlugin,
	CloseTab,
	CoverView,
	Minimize,
	Next,
	PlayNewTab,
	PlayNext,
	PlayPause,
	Prev,
	Quit,
	ReloadLibrary,
	SeekBwd,
	SeekBwdFast,
	SeekFwd,
	SeekFwdFast,
	Stop,
	TogglePlaylistLock,
	ViewLibrary,
	VolDown,
	VolUp,
	Invalid
};

#endif // SHORTCUTIDENTIFIER_H

